@@ -3,7 +3,8 @@
 uudecode isakmp-identification-segfault.puu
 
 echo -n test isakmp3...
-if (../tcpdump -t -v -n -r isakmp-identification-segfault.pcap | diff - isakmp3.out)
+../tcpdump -t -v -n -r isakmp-identification-segfault.pcap > isakmp3.new
+if diff isakmp3.new isakmp3.out
 then
 	echo passed.
 else
